@@ -86,6 +86,21 @@
 						bool representsReal(const MathStructure &vargs, bool allow_units = false) const;\
 						bool representsNonComplex(const MathStructure &vargs, bool allow_units = false) const;\
 					};
+					
+
+#define DECLARE_BUILTIN_FUNCTION_R3(x)	class x : public MathFunction { \
+					  public: \
+						int calculate(MathStructure &mstruct, const MathStructure &vargs, const EvaluationOptions &eo);  \
+						x(); \
+						x(const x *function) {set(function);} \
+						ExpressionItem *copy() const {return new x(this);} \
+						bool representsNumber(const MathStructure &vargs, bool allow_units = false) const;\
+						bool representsReal(const MathStructure &vargs, bool allow_units = false) const;\
+						bool representsNonComplex(const MathStructure &vargs, bool allow_units = false) const;\
+						bool representsComplex(const MathStructure &vargs, bool allow_units = false) const;\
+						bool representsNonZero(const MathStructure &vargs, bool allow_units = false) const;\
+					};
+										
 #define DECLARE_BUILTIN_FUNCTION_R1(x)	class x : public MathFunction { \
 					  public: \
 						int calculate(MathStructure &mstruct, const MathStructure &vargs, const EvaluationOptions &eo);  \
@@ -120,6 +135,9 @@ DECLARE_BUILTIN_FUNCTION(PermanentFunction)
 DECLARE_BUILTIN_FUNCTION(AdjointFunction)
 DECLARE_BUILTIN_FUNCTION(CofactorFunction)
 DECLARE_BUILTIN_FUNCTION(InverseFunction)
+DECLARE_BUILTIN_FUNCTION(MagnitudeFunction)
+DECLARE_BUILTIN_FUNCTION(HadamardFunction)
+DECLARE_BUILTIN_FUNCTION(EntrywiseFunction)
 
 DECLARE_BUILTIN_FUNCTION_R(FactorialFunction)
 DECLARE_BUILTIN_FUNCTION_R(DoubleFactorialFunction)
@@ -128,6 +146,7 @@ DECLARE_BUILTIN_FUNCTION(BinomialFunction)
 
 DECLARE_BUILTIN_FUNCTION(BitXorFunction)
 DECLARE_BUILTIN_FUNCTION_B(XorFunction)
+DECLARE_BUILTIN_FUNCTION(BitCmpFunction)
 DECLARE_BUILTIN_FUNCTION_B(OddFunction)
 DECLARE_BUILTIN_FUNCTION_B(EvenFunction)
 DECLARE_BUILTIN_FUNCTION(ShiftFunction)
@@ -174,7 +193,7 @@ DECLARE_BUILTIN_FUNCTION(ExpFunction)
 DECLARE_BUILTIN_FUNCTION_R(LogFunction)
 DECLARE_BUILTIN_FUNCTION(LognFunction)
 
-DECLARE_BUILTIN_FUNCTION(LambertWFunction)
+DECLARE_BUILTIN_FUNCTION_R3(LambertWFunction)
 
 DECLARE_BUILTIN_FUNCTION_R2(SinFunction)
 DECLARE_BUILTIN_FUNCTION_R2(CosFunction)
@@ -210,6 +229,7 @@ DECLARE_BUILTIN_FUNCTION(ModeFunction)
 DECLARE_BUILTIN_FUNCTION_RPI(RandFunction)
 
 DECLARE_BUILTIN_FUNCTION(DateFunction)
+DECLARE_BUILTIN_FUNCTION(DateTimeFunction)
 DECLARE_BUILTIN_FUNCTION(TimeValueFunction)
 DECLARE_BUILTIN_FUNCTION(TimestampFunction)
 DECLARE_BUILTIN_FUNCTION(TimestampToDateFunction)
@@ -226,6 +246,9 @@ DECLARE_BUILTIN_FUNCTION(AddDaysFunction)
 DECLARE_BUILTIN_FUNCTION(AddMonthsFunction)
 DECLARE_BUILTIN_FUNCTION(AddYearsFunction)
 
+DECLARE_BUILTIN_FUNCTION(LunarPhaseFunction)
+DECLARE_BUILTIN_FUNCTION(NextLunarPhaseFunction)
+
 DECLARE_BUILTIN_FUNCTION(BinFunction)
 DECLARE_BUILTIN_FUNCTION(OctFunction)
 DECLARE_BUILTIN_FUNCTION(HexFunction)
@@ -275,6 +298,7 @@ DECLARE_BUILTIN_FUNCTION(IntegrateFunction)
 DECLARE_BUILTIN_FUNCTION(SolveFunction)
 DECLARE_BUILTIN_FUNCTION(SolveMultipleFunction)
 DECLARE_BUILTIN_FUNCTION(DSolveFunction)
+DECLARE_BUILTIN_FUNCTION(LimitFunction)
 
 DECLARE_BUILTIN_FUNCTION_R2(liFunction)
 DECLARE_BUILTIN_FUNCTION_R2(LiFunction)
