@@ -1,7 +1,7 @@
 /*
     Qalculate    
 
-    Copyright (C) 2003-2007  Niklas Knutsson (nq@altern.org)
+    Copyright (C) 2003-2007, 2008, 2016-2018  Hanna Knutsson (hanna.knutsson@protonmail.com)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
 
 #include <libqalculate/includes.h>
 #include <libqalculate/util.h>
-#include <pthread.h>
+#include <sys/time.h>
 
 /** @file */
 
@@ -24,22 +24,31 @@
 *
 * libqalculate is math libary for expression evaluation with units, variables and functions support and CAS functionality.
 *
-* The main parts of the library is the almighty Calculator class, the MathStructure class for mathematical expressions and classes for objects in an epxression,
+* The main parts of the library is the almighty Calculator class, the MathStructure class for mathematical expressions and classes for objects in an expression,
 * mostly of the class Numbers and sub classes of ExpressionItem.
 *
-* A simple application using libqalculate need only create a calculator object, perhaps load definitions (functions, variables, units, etc.) and use the calculate function as follows:
-* \code new Calculator();
+* A simple application using libqalculate need only create a calculator object, perhaps load definitions (functions, variables, units, etc.), and calculate (and output) an expression as follows:
+* \code 
+* new Calculator();
 * CALCULATOR->loadGlobalDefinitions();
 * CALCULATOR->loadLocalDefinitions();
+* cout << CALCULATOR->calculateAndPrint("1 + 1", 2000) << endl;\endcode
+* In the above example, the calculation is terminated after two seconds (2000 ms), if it is not finished before then.
+* Applications using localized numbers should first call Calculalor::unlocalizeExpression() on the expression.
+*
+* A less simple application might calculate and output the expression separately.
+* \code
 * EvaluationOptions eo;
-* MathStructure result = CALCULATOR->calculate("1 + 1", eo);\endcode
+* MathStructure result;
+* CALCULATOR->calculate(&mstruct, unlocalizeExpression("1 + 1"), 2000, eo);\endcode
 *
 * More complex usage mainly involves manipulating objects of the MathStructure class directly.
 *
-* To display the resulting expression you will normally use MathStructure::format() followed by MathStructure::print() as follows:
-* \code PrintOptions po;
-* result.format(po);
-* string result_str = result.print(po); \endcode
+* To display the resulting expression use Calculator::print() as follows:
+* \code
+* PrintOptions po;
+* string result_str = CALCULATOR->print(result, 2000, po);\endcode
+* Alternatively MathStructure::format() followed by MathStructure::print() can be used, whithout the possiblity to specify a time limit.
 *
 * Central to the flexiblity of libqalculate is the many options passed to evaluating and display functions with EvaluationOptions and PrintOptions.
 *
@@ -50,9 +59,9 @@
 * For a simple program use pkg-config on the command line:
 * \code c++ `pkg-config --cflags --libs libqalculate` hello.c -o hello \endcode
 *
-* If the program uses autoconf, put the following in configure.in:
+* If the program uses autoconf, put the following in configure.ac:
 * \code PKG_CHECK_MODULES(QALCULATE, [
-*	libqalculate >= 0.9.7
+*	libqalculate >= 1.0.0
 *	])
 * AC_SUBST(QALCULATE_CFLAGS)
 * AC_SUBST(QALCULATE_LIBS) \endcode
@@ -123,6 +132,8 @@ struct PlotDataParameters {
 	bool yaxis2;
 	/// Use scale on second x-axis
 	bool xaxis2;
+	/// Check if data is continuous
+	bool test_continuous;
 	PlotDataParameters();
 };
 
@@ -133,17 +144,32 @@ typedef enum {
 	MESSAGE_ERROR
 } MessageType;
 
+///Message stages
+#define MESSAGE_STAGE_CONVERSION		-4
+#define MESSAGE_STAGE_CONVERSION_PARSING	-3
+#define MESSAGE_STAGE_CALCULATION		-2
+#define MESSAGE_STAGE_PARSING			-1
+#define MESSAGE_STAGE_UNSET			0
+
+///Message categories
+#define MESSAGE_CATEGORY_NONE		0
+#define MESSAGE_CATEGORY_PARSING	1
+
+
 /// A message with information to the user. Primarily used for errors and warnings.
 class CalculatorMessage {
   protected:
 	string smessage;
 	MessageType mtype;
+	int i_stage, i_cat;
   public:
-	CalculatorMessage(string message_, MessageType type_ = MESSAGE_WARNING);
+	CalculatorMessage(string message_, MessageType type_ = MESSAGE_WARNING, int cat_ = MESSAGE_CATEGORY_NONE, int stage_ = MESSAGE_STAGE_UNSET);
 	CalculatorMessage(const CalculatorMessage &e);
 	string message() const;
-	const char* c_message() const;	
+	const char* c_message() const;
 	MessageType type() const;
+	int stage() const;
+	int category() const;
 };
 
 #include <libqalculate/MathStructure.h>
@@ -172,6 +198,8 @@ struct Element {
 
 #define UFV_LENGTHS	20
 
+class Calculate_p;
+
 /// The almighty calculator class.
 /** The calculator class is responsible for loading functions, variables and units, and keeping track of them, as well as parsing expressions and much more. A calculator object must be created before any other Qalculate! class is used. There should never be more than one calculator object, accessed with CALCULATOR. 
 *
@@ -189,6 +217,9 @@ class Calculator {
 
 	int ianglemode;
 	int i_precision;
+	bool b_interval;
+	int i_stop_interval;
+	int i_start_interval;
 	char vbuffer[200];
 	vector<void*> ufvl;
 	vector<char> ufvl_t;
@@ -198,107 +229,118 @@ class Calculator {
 	
 	vector<DataSet*> data_sets;
 	
-	Sgi::hash_map<size_t, MathStructure*> id_structs;
-	Sgi::hash_map<size_t, bool> ids_p;
-	vector<size_t> freed_ids;	
-	size_t ids_i;
+	class Calculator_p *priv;
 	
-	vector<string> signs;	
+	vector<string> signs;
 	vector<string> real_signs;
-	vector<string> default_signs;	
-	vector<string> default_real_signs;	
+	vector<string> default_signs;
+	vector<string> default_real_signs;
 	char *saved_locale;
 	int disable_errors_ref;
 	vector<int> stopped_errors_count;
 	vector<int> stopped_warnings_count;
 	vector<int> stopped_messages_count;
-	pthread_t calculate_thread;
-	pthread_attr_t calculate_thread_attr;
-	pthread_t print_thread;
-	pthread_attr_t print_thread_attr;
-	bool b_functions_was, b_variables_was, b_units_was, b_unknown_was, b_calcvars_was, b_rpn_was;
+	vector<vector<CalculatorMessage> > stopped_messages;
+
+	Thread *calculate_thread;
+
 	string NAME_NUMBER_PRE_S, NAME_NUMBER_PRE_STR, DOT_STR, DOT_S, COMMA_S, COMMA_STR, ILLEGAL_IN_NAMES, ILLEGAL_IN_UNITNAMES, ILLEGAL_IN_NAMES_MINUS_SPACE_STR;
 
 	bool b_argument_errors;
-	bool exchange_rates_warning_issued;
+	int current_stage;
 
-	int has_gnomevfs;
+	time_t exchange_rates_time[3], exchange_rates_check_time[3];
+	int b_exchange_rates_used;
+	bool b_exchange_rates_warning_enabled;
 
 	bool b_gnuplot_open;
 	string gnuplot_cmdline;
-	FILE *gnuplot_pipe, *calculate_pipe_r, *calculate_pipe_w, *print_pipe_r, *print_pipe_w;
+	FILE *gnuplot_pipe;
 	
 	bool local_to;
 	
 	Assumptions *default_assumptions;
 	
 	vector<Variable*> deleted_variables;
-	vector<MathFunction*> deleted_functions;	
+	vector<MathFunction*> deleted_functions;
 	vector<Unit*> deleted_units;
 	
+	bool b_var_units;
+	
 	bool b_save_called;
 	
+	int i_timeout;
+	struct timeval t_end;
+	int i_aborted;
+	bool b_controlled;
+	
 	string per_str, times_str, plus_str, minus_str, and_str, AND_str, or_str, OR_str, XOR_str;
 	size_t per_str_len, times_str_len, plus_str_len, minus_str_len, and_str_len, AND_str_len, or_str_len, OR_str_len, XOR_str_len;
 
 	vector<MathStructure*> rpn_stack;
 
-	bool calculateRPN(MathStructure *mstruct, int command, size_t index, int msecs, const EvaluationOptions &eo);
-	bool calculateRPN(string str, int command, size_t index, int msecs, const EvaluationOptions &eo, MathStructure *parsed_struct, MathStructure *to_struct, bool make_to_division);
+	bool calculateRPN(MathStructure *mstruct, int command, size_t index, int msecs, const EvaluationOptions &eo, int function_arguments = 0);
+	bool calculateRPN(string str, int command, size_t index, int msecs, const EvaluationOptions &eo, MathStructure *parsed_struct, MathStructure *to_struct, bool make_to_division, int function_arguments = 0);
 	
   public:
 
-	KnownVariable *v_pi, *v_e, *v_i, *v_inf, *v_pinf, *v_minf, *v_undef;
+	KnownVariable *v_pi, *v_e, *v_euler, *v_catalan, *v_i, *v_pinf, *v_minf, *v_undef, *v_precision;
+	KnownVariable *v_percent, *v_permille, *v_permyriad;
+	KnownVariable *v_today, *v_yesterday, *v_tomorrow, *v_now;
 	UnknownVariable *v_x, *v_y, *v_z;
+	UnknownVariable *v_C, *v_n;
 	MathFunction *f_vector, *f_sort, *f_rank, *f_limits, *f_component, *f_dimension, *f_merge_vectors;
-	MathFunction *f_matrix, *f_matrix_to_vector, *f_area, *f_rows, *f_columns, *f_row, *f_column, *f_elements, *f_element, *f_transpose, *f_identity, *f_determinant, *f_permanent, *f_adjoint, *f_cofactor, *f_inverse; 
+	MathFunction *f_matrix, *f_matrix_to_vector, *f_area, *f_rows, *f_columns, *f_row, *f_column, *f_elements, *f_element, *f_transpose, *f_identity, *f_determinant, *f_permanent, *f_adjoint, *f_cofactor, *f_inverse, *f_magnitude, *f_hadamard, *f_entrywise; 
 	MathFunction *f_factorial, *f_factorial2, *f_multifactorial, *f_binomial;
-	MathFunction *f_xor, *f_bitxor, *f_even, *f_odd, *f_shift;
-	MathFunction *f_abs, *f_gcd, *f_lcm, *f_signum, *f_round, *f_floor, *f_ceil, *f_trunc, *f_int, *f_frac, *f_rem, *f_mod;	
+	MathFunction *f_xor, *f_bitxor, *f_even, *f_odd, *f_shift, *f_bitcmp;
+	MathFunction *f_abs, *f_gcd, *f_lcm, *f_signum, *f_heaviside, *f_dirac, *f_round, *f_floor, *f_ceil, *f_trunc, *f_int, *f_frac, *f_rem, *f_mod;	
 	MathFunction *f_polynomial_unit, *f_polynomial_primpart, *f_polynomial_content, *f_coeff, *f_lcoeff, *f_tcoeff, *f_degree, *f_ldegree;
-	MathFunction *f_re, *f_im, *f_arg, *f_numerator, *f_denominator;
-  	MathFunction *f_sqrt, *f_sq;
+	MathFunction *f_re, *f_im, *f_arg, *f_numerator, *f_denominator, *f_interval;
+  	MathFunction *f_sqrt, *f_cbrt, *f_root, *f_sq;
 	MathFunction *f_exp;
 	MathFunction *f_ln, *f_logn;
 	MathFunction *f_lambert_w;
-	MathFunction *f_sin, *f_cos, *f_tan, *f_asin, *f_acos, *f_atan, *f_sinh, *f_cosh, *f_tanh, *f_asinh, *f_acosh, *f_atanh, *f_radians_to_default_angle_unit;
-	MathFunction *f_zeta, *f_gamma, *f_beta;
+	MathFunction *f_sin, *f_cos, *f_tan, *f_asin, *f_acos, *f_atan, *f_sinh, *f_cosh, *f_tanh, *f_asinh, *f_acosh, *f_atanh, *f_atan2, *f_sinc, *f_radians_to_default_angle_unit;
+	MathFunction *f_zeta, *f_gamma, *f_digamma, *f_beta, *f_airy, *f_besselj, *f_bessely, *f_erf, *f_erfc;
 	MathFunction *f_total, *f_percentile, *f_min, *f_max, *f_mode, *f_rand;
-	MathFunction *f_isodate, *f_localdate, *f_timestamp, *f_stamptodate, *f_days, *f_yearfrac, *f_week, *f_weekday, *f_month, *f_day, *f_year, *f_yearday, *f_time, *f_add_days, *f_add_months, *f_add_years;
+	MathFunction *f_date, *f_datetime, *f_timevalue, *f_timestamp, *f_stamptodate, *f_days, *f_yearfrac, *f_week, *f_weekday, *f_month, *f_day, *f_year, *f_yearday, *f_time, *f_add_days, *f_add_months, *f_add_years;
+	MathFunction *f_lunarphase, *f_nextlunarphase;
 	MathFunction *f_bin, *f_oct, *f_hex, *f_base, *f_roman;
 	MathFunction *f_ascii, *f_char;
 	MathFunction *f_length, *f_concatenate;
 	MathFunction *f_replace, *f_stripunits;
 	MathFunction *f_genvector, *f_for, *f_sum, *f_product, *f_process, *f_process_matrix, *f_csum, *f_if, *f_is_number, *f_is_real, *f_is_rational, *f_is_integer, *f_represents_number, *f_represents_real, *f_represents_rational, *f_represents_integer, *f_function, *f_select;
-	MathFunction *f_diff, *f_integrate, *f_solve, *f_multisolve;
+	MathFunction *f_diff, *f_integrate, *f_solve, *f_multisolve, *f_dsolve, *f_limit;
+	MathFunction *f_li, *f_Li, *f_Ei, *f_Si, *f_Ci, *f_Shi, *f_Chi, *f_igamma;
 	MathFunction *f_error, *f_warning, *f_message, *f_save, *f_load, *f_export, *f_title;
 	MathFunction *f_register, *f_stack;
-	MathFunction *f_uncertainty;
+	MathFunction *f_plot;
 	
-	Unit *u_rad, *u_gra, *u_deg, *u_euro;
+	Unit *u_rad, *u_gra, *u_deg, *u_euro, *u_btc, *u_second, *u_minute, *u_hour, *u_year, *u_month, *u_day;
 	DecimalPrefix *decimal_null_prefix;
 	BinaryPrefix *binary_null_prefix;
 
   	bool place_currency_code_before, place_currency_sign_before;
 	bool place_currency_code_before_negative, place_currency_sign_before_negative;
 	bool default_dot_as_separator;
+	string local_digit_group_separator, local_digit_group_format;
   
-  	bool b_busy, calculate_thread_stopped, print_thread_stopped;
-	string expression_to_calculate, tmp_print_result;
-	PrintOptions tmp_printoptions;
+  	bool b_busy;
+	string expression_to_calculate;
 	EvaluationOptions tmp_evaluationoptions;
 	MathStructure *tmp_parsedstruct;
 	MathStructure *tmp_tostruct;
 	MathStructure *tmp_rpn_mstruct;
 	bool tmp_maketodivision;
 	int tmp_proc_command;
+	int tmp_proc_registers;
 	size_t tmp_rpnindex;
 	
-	PrintOptions save_printoptions;	
+	PrintOptions save_printoptions, message_printoptions;
   
 	vector<Variable*> variables;
-	vector<MathFunction*> functions;	
-	vector<Unit*> units;	
+	vector<MathFunction*> functions;
+	vector<Unit*> units;
 	vector<Prefix*> prefixes;
 	vector<DecimalPrefix*> decimal_prefixes;
 	vector<BinaryPrefix*> binary_prefixes;
@@ -320,7 +362,7 @@ class Calculator {
 	* @param msecs The maximum time for the calculation in milliseconds. If msecs <= 0 the time will be unlimited.
 	* @param eo Options for the evaluation and parsing of the expression.
 	* @param[out] parsed_struct NULL or a math structure to fill with the result of the parsing of the expression.
-	* @param[out] to_struct NULL or a math structure to fill with unit expression parsed after "to".
+	* @param[out] to_struct NULL or a math structure to fill with unit expression parsed after "to". If expression does not contain a "to" string, and to_struct is a unit or a symbol (a unit expression string), to_struct will be used instead.
 	* @param make_to_division If true, the expression after "to" will be interpreted as a unit epxression to convert the result to.
 	* @returns true if the calculation was successfully started (and finished if msecs > 0).
 	*/
@@ -330,26 +372,94 @@ class Calculator {
 	* @param str Expression.
 	* @param eo Options for the evaluation and parsing of the expression.
 	* @param[out] parsed_struct NULL or a math structure to fill with the result of the parsing of the expression.
-	* @param[out] to_struct NULL or a math structure to fill with unit expression parsed after "to".
+	* @param[out] to_struct NULL or a math structure to fill with unit expression parsed after "to". If expression does not contain a "to" string, and to_struct is a unit or a symbol (a unit expression string), to_struct will be used instead.
 	* @param make_to_division If true, the expression after "to" will be interpreted as a unit epxression to convert the result to.
 	* @returns The result of the calculation.
 	*/
 	MathStructure calculate(string str, const EvaluationOptions &eo = default_evaluation_options, MathStructure *parsed_struct = NULL, MathStructure *to_struct = NULL, bool make_to_division = true);
-	string printMathStructureTimeOut(const MathStructure &mstruct, int msecs = 100000, const PrintOptions &op = default_print_options);
+	/** Calculates a parsed value.
+	* This function starts the calculation in a separate thread and will return when the calculation has started unless a maximum time has been specified.
+	* The calculation can then be stopped with abort().
+	*
+	* @param[out] mstruct Parsed value to evaluate and fill with the result.
+	* @param msecs The maximum time for the calculation in milliseconds. If msecs <= 0 the time will be unlimited.
+	* @param eo Options for the evaluation of the expression.
+	* @param to_str "to" expression for conversion.
+	* @returns The result of the calculation.
+	*/
+	bool calculate(MathStructure *mstruct, int msecs, const EvaluationOptions &eo = default_evaluation_options, string to_str = "");
+	/** Calculates a parsed value.
+	*
+	* @param mstruct Parsed value to evaluate.
+	* @param eo Options for the evaluation of the expression.
+	* @param to_str "to" expression for conversion.
+	* @returns The result of the calculation.
+	*/
+	MathStructure calculate(const MathStructure &mstruct, const EvaluationOptions &eo = default_evaluation_options, string to_str = "");
+	/** Calculates an expression.and outputs the result to a text string. The expression should be unlocalized first with unlocalizeExpression().
+	* 
+	* Unlike other functions for expression evaluation this function handles ending "to"-commmands, in addition to unit conversion, such "to hexadecimal" or to "fractions", similar to the qalc application.
+	* 
+	*
+	* @param str Expression.
+	* @param msecs The maximum time for the calculation in milliseconds. If msecs <= 0 the time will be unlimited.
+	* @param eo Options for the evaluation and parsing of the expression.
+	* @param po Result formatting options.
+	* @returns The result of the calculation.
+	* \since 2.6.0
+	*/
+	string calculateAndPrint(string str, int msecs = 10000, const EvaluationOptions &eo = default_evaluation_options, const PrintOptions &po = default_print_options);
 	int testCondition(string expression);
 	//@}
 
+	/** @name Functions for printing expressions with the option to set maximum time or abort. */
+	//@{
+	/** Calls MathStructure::format(po) and MathStructure::print(po). The process is aborted after msecs milliseconds.
+	*/
+	string print(const MathStructure &mstruct, int milliseconds = 100000, const PrintOptions &op = default_print_options);
+	///Deprecated: use print() instead
+	string printMathStructureTimeOut(const MathStructure &mstruct, int milliseconds = 100000, const PrintOptions &op = default_print_options);
+	
+	/** Called before calculation, formatting or printing of a MathStructure (Calculator::calculate(), without maximum time, MathStructure::eval(), MathStructure::format() and MathStructure::print(), etc.) or printing of a Number (using Number::print) to be able to abort the process. Always use Calculator::stopControl() after finishing.
+	*
+	* @param milliseconds The maximum time for the process in milliseconds. If msecs <= 0 the time will be unlimited (stop with abort()).
+	*/
+	void startControl(int milliseconds = 0);
+	/** Always call this function after Calculator::startControl() after formatting, printing or calculation has finished.
+	*/
+	void stopControl(void);
+	/** Abort formatting, printing or evaluation (after startControl() has been called).
+	* This function will normally be called from a thread that checks for user input or other conditions.
+	*
+	* @returns false if the calculation thread was forcibly stopped
+	*/
+	bool abort();
+	bool aborted(void);
+	bool isControlled(void) const;
+	string abortedMessage(void) const;
+	string timedOutString(void) const;
+	
+	///Deprecated: use startControl()
+	void startPrintControl(int milliseconds = 0);
+	///Deprecated: use abort()
+	void abortPrint(void);
+	///Deprecated: use stopControl()
+	void stopPrintControl(void);
+	/// Deprecated: use aborted()
+	bool printingAborted(void);
+	/// Deprecated: use isControlled()
+	bool printingControlled(void) const;
+	/// Deprecated: use abortedMessage()
+	string printingAbortedMessage(void) const;
+	//@}
+
 	/** @name Functions for handling of threaded calculations */
 	//@{
-	/** Aborts the current calculation. */
-	void abort();
-	/** Aborts the current calculation. Used from within the calculation thread. */
-	void abort_this();
 	/** Returns true if the calculate or print thread is busy. */
 	bool busy();
-	/** Saves the state of the calculator. Used internally to be able to restore the state after aborted calculation. */
+	/// Deprecated: does nothing.
 	void saveState();
-	/** Restores the saved state of the calculator. Used internally to restore the state after aborted calculation. */
+	/// Deprecated: does nothing.
 	void restoreState();
 	/** Clears all stored values. Used internally after aborted calculation. */
 	void clearBuffers();
@@ -509,7 +619,7 @@ class Calculator {
 	* @param str The expression to localize.
 	* @returns A localized expression.
 	*/
-	string localizeExpression(string str) const;
+	string localizeExpression(string str, const ParseOptions &po = default_parse_options) const;
 	/** Returns an unlocalized expressions. Affects decimal signs and argument separators.
 	*
 	* @param str The expression to unlocalize.
@@ -523,9 +633,10 @@ class Calculator {
 	* @param eo Options for the evaluation and parsing of the expression (nothing will be done if units are not enabled).
 	* @returns true if " to " was found and the expression split.
 	*/
-	bool separateToExpression(string &str, string &to_str, const EvaluationOptions &eo) const;
+	bool separateToExpression(string &str, string &to_str, const EvaluationOptions &eo, bool keep_modifiers = false, bool allow_empty_from = false) const;	
+	bool hasToExpression(const string &str, bool allow_empty_from = false) const;
 
-	void parseSigns(string &str) const;
+	void parseSigns(string &str, bool convert_to_internal_representation = false) const;
 	/** Parse an expression and place in a MathStructure object.
 	*
 	* @param str Expression
@@ -536,7 +647,7 @@ class Calculator {
 	void parse(MathStructure *mstruct, string str, const ParseOptions &po = default_parse_options);
 	bool parseNumber(MathStructure *mstruct, string str, const ParseOptions &po = default_parse_options);
 	bool parseOperators(MathStructure *mstruct, string str, const ParseOptions &po = default_parse_options);
-	bool parseAdd(string &str, MathStructure *mstruct, const ParseOptions &po, MathOperation s);
+	bool parseAdd(string &str, MathStructure *mstruct, const ParseOptions &po, MathOperation s, bool append = true);
 	bool parseAdd(string &str, MathStructure *mstruct, const ParseOptions &po);
 	//@}
 
@@ -548,9 +659,10 @@ class Calculator {
 	* @param mstruct The value to convert.
 	* @param composite_ Unit expression.
 	* @param eo Evaluation options.
+	* @param[out] units NULL or a math structure to fill with the parsed unit expression(or set to undefined if no units were found).
 	* @returns Converted value.
 	*/
-	MathStructure convert(const MathStructure &mstruct, string composite_, const EvaluationOptions &eo = default_evaluation_options);
+	MathStructure convert(const MathStructure &mstruct, string composite_, const EvaluationOptions &eo = default_evaluation_options, MathStructure *units = NULL);
 	/** Converts to a unit.
 	* The converted value is evaluated.
 	*
@@ -560,15 +672,23 @@ class Calculator {
 	* @param always_convert ...
 	* @returns Converted value.
 	*/
-	MathStructure convert(const MathStructure &mstruct, Unit *to_unit, const EvaluationOptions &eo = default_evaluation_options, bool always_convert = true);
+	MathStructure convert(const MathStructure &mstruct, Unit *to_unit, const EvaluationOptions &eo = default_evaluation_options, bool always_convert = true, bool convert_to_mixed_units = true);
+	MathStructure convert(const MathStructure &mstruct, KnownVariable *to_var, const EvaluationOptions &eo = default_evaluation_options);
 	MathStructure convert(double value, Unit *from_unit, Unit *to_unit, const EvaluationOptions &eo = default_evaluation_options);
-	MathStructure convert(string str, Unit *from_unit, Unit *to_unit, const EvaluationOptions &eo = default_evaluation_options);
+	MathStructure convert(string str, Unit *from_unit, Unit *to_unit, int milliseconds, const EvaluationOptions &eo = default_evaluation_options);
+	/** Depecated: use convert() */
+	MathStructure convertTimeOut(string str, Unit *from_unit, Unit *to_unit, int milliseconds, const EvaluationOptions &eo = default_evaluation_options);
+	MathStructure convert(string str, Unit *from_unit, Unit *to_unit, const EvaluationOptions &eo = default_evaluation_options);	
 	MathStructure convertToBaseUnits(const MathStructure &mstruct, const EvaluationOptions &eo = default_evaluation_options);
-	Unit *getBestUnit(Unit *u, bool allow_only_div = false);
-	MathStructure convertToBestUnit(const MathStructure &mstruct, const EvaluationOptions &eo = default_evaluation_options);
+	Unit *getBestUnit(Unit *u, bool allow_only_div = false, bool convert_to_local_currency = true);
+	MathStructure convertToBestUnit(const MathStructure &mstruct, const EvaluationOptions &eo = default_evaluation_options, bool convert_to_si_units = true);
 	MathStructure convertToCompositeUnit(const MathStructure &mstruct, CompositeUnit *cu, const EvaluationOptions &eo = default_evaluation_options, bool always_convert = true);
+	MathStructure convertToMixedUnits(const MathStructure &mstruct, const EvaluationOptions &eo = default_evaluation_options);
 	//@}
 
+	/** Used by the UI to find unit category for a mathematical expression.*/
+	Unit *findMatchingUnit(const MathStructure &mstruct);
+
 	/** @name Functions for default assumptions for unknown variables and symbols */
 	//@{
 	/** Set assumptions for objects without own assumptions (unknown variables and symbols).
@@ -703,6 +823,7 @@ class Calculator {
 	Unit* getUnit(string name_);
 	Unit* getActiveUnit(string name_);
 	Unit* getCompositeUnit(string internal_name_);
+	Unit* getLocalCurrency();
 	/** Returns prefix for an index (starting at zero). All prefixes can be traversed by starting at index zero and increasing the index until NULL is returned.
 	*
 	* @param index Index of prefix.
@@ -766,6 +887,9 @@ class Calculator {
 	/** Adds builtin units. Called automatically when the calculator is created. */
 	void addBuiltinUnits();
 	//@}
+	
+	void setVariableUnitsEnabled(bool enable_variable_units = true);
+	bool variableUnitsEnabled() const;
 
 	/** @name Functions for testing validity of functions, variable and unit names. */
 	//@{
@@ -810,10 +934,13 @@ class Calculator {
 
 	/** @name Functions for message handling. */
 	//@{
+	void error(bool critical, int message_category, const char *TEMPLATE,...);
 	void error(bool critical, const char *TEMPLATE,...);
 	/** Put a message in the message queue. 
 	*/
+	void message(MessageType mtype, int message_category, const char *TEMPLATE,...);
 	void message(MessageType mtype, const char *TEMPLATE,...);
+	void message(MessageType mtype, int message_category, const char *TEMPLATE, va_list ap);
 	/** Returns the first message in queue.
 	*/
 	CalculatorMessage *message();
@@ -822,7 +949,11 @@ class Calculator {
 	CalculatorMessage *nextMessage();
 	bool showArgumentErrors() const;
 	void beginTemporaryStopMessages();
-	int endTemporaryStopMessages(int *message_count = NULL, int *warning_count = NULL);	
+	int endTemporaryStopMessages(int *message_count = NULL, int *warning_count = NULL, int release_messages_if_no_equal_or_greater_than_message_type = -1);
+	void endTemporaryStopMessages(bool release_messages, vector<CalculatorMessage> *blocked_messages = NULL);
+	void addMessages(vector<CalculatorMessage> *message_vector);
+	const PrintOptions &messagePrintOptions() const;
+	void setMessagePrintOptions(const PrintOptions &po);
 	//@}
 
 	/** @name Functions for loading and saving definitions (variables, functions, units, etc.). */
@@ -887,8 +1018,9 @@ class Calculator {
 	*/
 	bool saveDefinitions();	
 	int saveDataObjects();
-	int savePrefixes(const char *file_name, bool save_global = false);	
-	int saveVariables(const char *file_name, bool save_global = false);	
+	int savePrefixes(const char *file_name, bool save_global = false);
+	string temporaryCategory(void) const;
+	int saveVariables(const char *file_name, bool save_global = false);
 	int saveUnits(const char *file_name, bool save_global = false);	
 	int saveFunctions(const char *file_name, bool save_global = false);
 	int saveDataSets(const char *file_name, bool save_global = false);
@@ -902,16 +1034,14 @@ class Calculator {
 	//@}
 	
 	/** @name Functions for exchange rates. */
-	//@{
-	/** Checks if gnomevfs-copy or wget is available for downloading exchange rates from the Internet.
+	/** Checks if able to downloading exchange rates from the Internet (using libcurl).
 	*
-	* @returns true if gnomevfs-copy or wget was found.
+	* @returns true if exchange rates can downloaded (if libcurl is available).
 	*/
 	bool canFetch();
-	/** Checks if gnomevfs-copy available.
-	*
-	* @returns true if gnomevfs-copy was found.
-	*/
+	///Deprecated: gvfs is not needed anymore.
+	bool hasGVFS();
+	///Deprecated: gvfs is not needed anymore.
 	bool hasGnomeVFS();
 	/** Load saved (local) currency units and exchange rates.
 	*
@@ -919,30 +1049,48 @@ class Calculator {
 	*/
 	bool loadExchangeRates();
 	/** Name of the exchange rates file on local disc.
+	* Multiple exchange rates sources might be used. Iterate over these, using the index parameter, until an empty string is returned.
 	*
+	* @param index The index (starting at one) of the exchange rate source
 	* @returns name of local exchange rates file.
 	*/
-	string getExchangeRatesFileName();
+	string getExchangeRatesFileName(int index = 1);
 	/** Url of the exchange rates file on the Internet.
+	* Multiple exchange rates sources might be used. Iterate over these, using the index parameter, until an empty string is returned.
 	*
+	* @param index The index (starting at one) of the exchange rate source
 	* @returns Url of exchange rates file.
 	*/
-	string getExchangeRatesUrl();
-	/** Download current exchange rates from the Internet to local disc.
+	string getExchangeRatesUrl(int index = 1);
+	/** Modification time of the exchange rates file.
 	*
-	* @param timeout Timeout for donwload try (only used by wget)
-	* @param wget_args Extra arguments to pass to wget.
-	* @returns true if operation was successful.
+	* @returns Returns exchange rates modification time.
 	*/
-	bool fetchExchangeRates(int timeout, string wget_args);
+	time_t getExchangeRatesTime(int index = -1);
+	///Deprecated: wget arguments are not used
+	bool fetchExchangeRates(int seconds, string wget_args);
 	/** Download current exchange rates from the Internet to local disc with default wget arguments.
 	*
-	* @param timeout Timeout for donwload try (only used by wget)
+	* @param seconds Maximum time for donwload try
 	* @returns true if operation was successful.
 	*/
-	bool fetchExchangeRates(int timeout = 15);
-	/** Returns true if the exchange rates on local disc is older than one week. */
-	bool checkExchangeRatesDate();
+	bool fetchExchangeRates(int seconds = 15, int n = -1);
+	/** Check age of exchange rates on local disc. 
+	*
+	* @param n_days How old in days exchange rates may be before exchange rates need updating
+	* @param force_check If exchange rates date should be checked again even if found outdated within n_days before
+	* @param send_warning If the standard exchange rates warning should be sent.
+	* @returns false if exchange.rates need updating
+	*/
+	bool checkExchangeRatesDate(unsigned int n_days = 7, bool force_check = false, bool send_warning = false, int n = -1);
+	/// Enable or disable old exchange rates warning (initial state is true).
+	void setExchangeRatesWarningEnabled(bool enable);
+	bool exchangeRatesWarningEnabled() const;
+	/// Check if exchange rates has been used since resetExchangeRatesUsed() was last called
+	int exchangeRatesUsed() const;
+	void resetExchangeRatesUsed();
+	/// For internal use, called by currency units
+	void setExchangeRatesUsed(int index);
 	//@}
 
 	/** @name Functions for plotting */
@@ -952,12 +1100,12 @@ class Calculator {
 	* @returns true if gnuplot was found.
 	*/
 	bool canPlot();
-	MathStructure expressionToPlotVector(string expression, const MathStructure &min, const MathStructure &max, int steps, MathStructure *x_vector = NULL, string x_var = "\\x", const ParseOptions &po = default_parse_options);
-	MathStructure expressionToPlotVector(string expression, float min, float max, int steps, MathStructure *x_vector = NULL, string x_var = "\\x", const ParseOptions &po = default_parse_options);
-	MathStructure expressionToPlotVector(string expression, const MathStructure &min, const MathStructure &max, const MathStructure &step, MathStructure *x_vector = NULL, string x_var = "\\x", const ParseOptions &po = default_parse_options);
-	MathStructure expressionToPlotVector(string expression, float min, float max, float step, MathStructure *x_vector = NULL, string x_var = "\\x", const ParseOptions &po = default_parse_options);
-	MathStructure expressionToPlotVector(string expression, const MathStructure &x_vector, string x_var = "\\x", const ParseOptions &po = default_parse_options);
-	bool plotVectors(PlotParameters *param, const vector<MathStructure> &y_vectors, const vector<MathStructure> &x_vectors, vector<PlotDataParameters*> &pdps, bool persistent = false);
+	MathStructure expressionToPlotVector(string expression, const MathStructure &min, const MathStructure &max, int steps, MathStructure *x_vector = NULL, string x_var = "\\x", const ParseOptions &po = default_parse_options, int msecs = 5000);
+	MathStructure expressionToPlotVector(string expression, float min, float max, int steps, MathStructure *x_vector = NULL, string x_var = "\\x", const ParseOptions &po = default_parse_options, int msecs = 5000);
+	MathStructure expressionToPlotVector(string expression, const MathStructure &min, const MathStructure &max, const MathStructure &step, MathStructure *x_vector = NULL, string x_var = "\\x", const ParseOptions &po = default_parse_options, int msecs = 5000);
+	MathStructure expressionToPlotVector(string expression, float min, float max, float step, MathStructure *x_vector = NULL, string x_var = "\\x", const ParseOptions &po = default_parse_options, int msecs = 5000);
+	MathStructure expressionToPlotVector(string expression, const MathStructure &x_vector, string x_var = "\\x", const ParseOptions &po = default_parse_options, int msecs = 5000);
+	bool plotVectors(PlotParameters *param, const vector<MathStructure> &y_vectors, const vector<MathStructure> &x_vectors, vector<PlotDataParameters*> &pdps, bool persistent = false, int msecs = 5000);
 	bool invokeGnuplot(string commands, string commandline_extra = "", bool persistent = false);
 	bool closeGnuplot();
 	bool gnuplotOpen();
@@ -973,6 +1121,18 @@ class Calculator {
 	/** Returns default precision for approximate calculations.
 	*/
 	int getPrecision() const;
+	/** Set if interval arithmetic should be used for approximate calculations.
+	*
+	* @param use_interval_arithmetic Set true to activate, or false to deactivate, interval arithmetic.
+	*/
+	void useIntervalArithmetic(bool use_interval_arithmetic = true);
+	/** Returns true if interval arithmetic are activated.
+	*/
+	bool usesIntervalArithmetic() const;
+	void beginTemporaryStopIntervalArithmetic();
+	void endTemporaryStopIntervalArithmetic();
+	void beginTemporaryEnableIntervalArithmetic();
+	void endTemporaryEnableIntervalArithmetic();
 	//@}
 
 	/** @name Functions for localization */
@@ -981,15 +1141,18 @@ class Calculator {
 	*/
 	const string &getDecimalPoint() const;
 	/** Returns the preferred comma character for separating arguments.*/
-	const string &getComma() const;	
+	const string &getComma() const;
 	/** Sets argument separator and decimal sign from the current locale. Mainly for internal use. */
 	void setLocale();
 	void useDecimalComma();
-	void useDecimalPoint();
+	/** Use point as decimal separator. 
+	* To use comma as an ignored separator in numbers, must be invoked with comma_as_separator = true, to change the default function argument separator to semicolon, in addition to using ParseOptions::comma_as_separator.
+	*/	
+	void useDecimalPoint(bool comma_as_separator = false);
 	/** Resets argument separator and decimal sign. Mainly for internal use. */
 	void unsetLocale();
 	/** Returns the translated text string used in expressions for converting to a specific unit expression (ex "5 meters to feet.*/
-	string localToString() const;
+	string localToString(bool include_spaces = true) const;
 	//@}
 
 	/** @name Functions adding alternative symbols for operators and such */
