@@ -1,7 +1,7 @@
 /*
     Qalculate    
 
-    Copyright (C) 2003-2007, 2008, 2016  Hanna Knutsson (hanna_k@fmgirl.com)
+    Copyright (C) 2003-2007, 2008, 2016  Hanna Knutsson (hanna.knutsson@protonmail.com)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -35,6 +35,8 @@ struct ExpressionName {
 	bool avoid_input;
 	/// If the name is case sensitive. The default behavior is that abbreviations are case sensitive and other names are not.
 	bool case_sensitive;
+	/// Use only for completion (useful for unicode letter alternatives)
+	bool completion_only;
 	/// The name.
 	string name;
 	
