@@ -1,14 +1,19 @@
 #include <libqalculate/qalculate.h>
 
-void display_errors() {
-	if(!CALCULATOR->message()) return;
+bool display_errors(bool show_only_errors = false) {
+	if(!CALCULATOR->message()) return false;
+	bool b_ret = false;
 	while(true) {
 		MessageType mtype = CALCULATOR->message()->type();
-		if(mtype == MESSAGE_ERROR) cout << "error: ";
-		else if(mtype == MESSAGE_WARNING) cout << "warning: ";
-		cout << CALCULATOR->message()->message() << endl;
+		if(!show_only_errors || mtype == MESSAGE_ERROR) {
+			if(mtype == MESSAGE_ERROR) cout << "error: ";
+			else if(mtype == MESSAGE_WARNING) cout << "warning: ";
+			cout << CALCULATOR->message()->message() << endl;
+			b_ret = true;
+		}
 		if(!CALCULATOR->nextMessage()) break;
 	}
+	return b_ret;
 }
 void clear_errors() {
 	if(!CALCULATOR->message()) return;
@@ -831,6 +836,581 @@ void test_intervals(bool use_interval) {
 
 }
 
+string rnd_expression(bool allow_unknowns, bool allow_functions, int length_factor1 = 10, int length_factor2 = 5, bool allow_units = false, bool allow_variables = false);
+
+
+string rnd_unit() {
+	int r = rand() % CALCULATOR->units.size();
+	Unit *u = CALCULATOR->units[r];
+	if(u->subtype() == SUBTYPE_COMPOSITE_UNIT) return ((CompositeUnit*) u)->print(false, true);
+	string str;
+	if(rand() % 3 == 0) {
+		r = rand() % CALCULATOR->prefixes.size();
+		str += CALCULATOR->prefixes[r]->name();
+	}
+	str += u->referenceName();
+	if(rand() % 3 == 0) {
+		str += "^";
+		str += ('2' + rand() % 2);
+	}
+	return str;
+}
+
+string rnd_var() {
+	int r = rand() % CALCULATOR->variables.size();
+	return CALCULATOR->variables[r]->name();
+}
+
+
+string rnd_number(bool use_par = true, bool only_integers = false, bool only_positive = false, bool allow_complex = true) {
+	string str;
+	bool par = false;
+	bool dot = only_integers;
+	bool started = false;
+	if(!only_positive && rand() % 3 == 0) {str += '-'; par = true;}
+	while(true) {
+		int r = rand();
+		if(!started) r = r % (only_positive ? 9 + 1 : 10 + 1);
+		else if(str.back() == '.') r = r % 10;
+		else r = r % ((dot ? 19 : 20) + str.length() * 10);
+		if(r > (dot ? 9 : 15)) break;
+		if((r >= 10 && r <= 15) || (!dot && !started && r == 0)) {if(!started) str += '0'; str += '.'; dot = true;}
+		else str += char('0' + r);
+		started = true;
+	}
+	if(allow_complex && !only_integers && rand() % 10 == 0) {str += 'i'; par = true;}
+	if(par && use_par) {str += ')'; str.insert(0, "(");}
+	return str;
+}
+
+string rnd_item(int &par, bool allow_function = true, int allow_unknown = 1, int allow_unit = false, int allow_variable = false) {
+	int r = rand() % (2 + (allow_unknown > 0)) + 1;
+	string str;
+	if(r != 1 || (!allow_unknown && !allow_function && !allow_unit && !allow_variable)) {
+		str = rnd_number();
+	} else {
+		if(allow_unit && (rand() % (2 + allow_function + allow_unknown + allow_variable)) == 0) {
+			str = rnd_unit();
+		} else if(allow_variable && (rand() % (2 + allow_function + allow_unknown)) == 0) {
+			if(!allow_function && !allow_unknown && !allow_variable && rand() % 2 == 0) str = rnd_number();
+			else str = rnd_var();
+		} else {
+			if(!allow_unknown) {
+				if(allow_function) r = rand() % 20 + 4;
+				else r = rand() % 2 + 4;
+			} else {
+				int au2 = 3 - allow_unknown % 3;
+				r = (rand() % ((allow_function ? 22 : 5) - au2)) + 4 - allow_unknown;
+				if(r < 4 - allow_unknown % 3) {
+					if(r < 0) r = -r;
+					if(allow_unknown % 3 == 1) r = 3;
+					else if(allow_unknown % 3 == 2) r = 2 + r % 2;
+					else r = 1 + r % 3;
+				}
+			}
+			switch(r) {
+				case 1: {str = "z"; break;}
+				case 2: {str = "y"; break;}
+				case 3: {str = "x"; break;}
+				case 4: {str = "pi"; break;}
+				case 5: {str = "e"; break;}
+				case 6: {str = "root("; 
+					str += rnd_expression(allow_unknown, allow_function, 6, 3, allow_unit, allow_variable);
+					str += ',';
+					str += rnd_number(true, true, true, false);
+					str += ')';
+					return str;
+				}
+				case 7: {str = "log("; 
+					str += rnd_expression(allow_unknown, allow_function, 6, 3, allow_unit, allow_variable);
+					str += ',';
+					str += rnd_number(true, true, true, false);
+					str += ')';
+					return str;
+				}
+				case 8: {str = "sin("; break;}
+				case 9: {str = "cos("; break;}
+				case 10: {str = "tan("; break;}
+				case 11: {str = "sinh("; break;}
+				case 12: {str = "cosh("; break;}
+				case 13: {str = "tanh("; break;}
+				case 14: {str = "asin("; break;}
+				case 15: {str = "acos("; break;}
+				case 16: {str = "atan("; break;}
+				case 17: {str = "asinh("; break;}
+				case 18: {str = "acosh("; break;}
+				case 19: {str = "atanh("; break;}
+				case 20: {str = "ln("; break;}
+				case 21: {str = "abs("; break;}
+				case 22: {str = "sqrt("; break;}
+				case 23: {str = "cbrt("; break;}
+				case 24: {str = "erf("; break;}
+				case 25: {str = "erfc("; break;}
+				case 26: {str = "airy("; break;}
+				case 27: {str = "Si("; break;}
+				case 28: {str = "Shi("; break;}
+				case 29: {str = "im("; break;}
+				case 30: {str = "re("; break;}
+			}
+			if(r > 5) {
+				if(allow_unknown && rand() % 2 == 1) {
+					str += "x)";
+					if(rand() % 3 == 1) str += "^2";
+				} else {
+					str += rnd_item(par, true, allow_unknown, allow_unit, allow_variable);
+					par++;
+				}
+			}
+		}
+	}
+	r = rand() % (5 * (par + 1)) + 1;
+	if(r == 1) {
+		str.insert(0, "(");
+		par++;
+	}
+	return str;
+}
+string rnd_operator(int &par, bool allow_pow = true) {
+	int r = rand() % (par ? 10 : 5) + 1;
+	string str;
+	if(par > 0 && r > 5) {
+		par--;
+		str = ")";
+		r -= 5;
+	}
+	if(r == 5 && !allow_pow) r = rand() % 4 + 1;
+	switch(r) {
+		case 1: return str + "+";
+		case 2: return str + "-";
+		case 3: return str + "*";
+		case 4: return str + "/";
+		case 5: {
+			switch(rand() % 4 + 1) {
+				case 1: return str + "^";
+				case 2: return str + "^2" + rnd_operator(par, false);
+				case 3: return str + "^3" + rnd_operator(par, false);
+				case 4: return str + "^-1" + rnd_operator(par, false);
+			}
+		}
+	}
+	return "";
+}
+
+string rnd_expression(bool allow_unknowns, bool allow_functions, int length_factor1, int length_factor2, bool allow_unit, bool allow_variable) {
+	int par = 0;
+	string str;
+	while(str.empty() || rand() % ((length_factor1 - (int) str.length() / length_factor2 < 2) ? 2 : (length_factor1 - (int) str.length() / length_factor2)) != 0) {
+		str += rnd_item(par, allow_functions, allow_unknowns, allow_unit, allow_variable);
+		str += rnd_operator(par);
+	}
+	if(str.back() != ')') str += rnd_item(par, false, allow_unknowns, allow_unit, allow_variable);
+	while(par > 0) {
+		str += ")";
+		par--;
+	}
+	return str;
+}
+
+KnownVariable *v;
+
+int rt1 = 0, rt2 = 0, rt3 = 0, rt4 = 0, rt5 = 0, rt6 = 0, rt7 = 0, rt8 = 0, rt9 = 0;
+void rnd_test(EvaluationOptions eo, int allow_unknowns, bool allow_functions, bool test_interval = true, bool test_equation = true, bool allow_unit = false, bool allow_variable = false) {
+	bool b_iv = CALCULATOR->usesIntervalArithmetic();
+	cerr << "A0" << endl;
+	string str = rnd_expression(allow_unknowns, allow_functions, 8, 4, allow_unit, allow_variable);
+	cerr << "A2:" << str << endl;
+	PrintOptions po; po.interval_display = INTERVAL_DISPLAY_SIGNIFICANT_DIGITS;
+	MathStructure mp, m1, m2;
+	CALCULATOR->parse(&mp, str, eo.parse_options);
+	eo.approximation = APPROXIMATION_APPROXIMATE;
+	m1 = mp;
+	cerr << "A3" << endl;
+	cerr << mp << endl;
+	CALCULATOR->calculate(&m1, 5000, eo);
+	if(m1.isAborted()) {cout << str << " => " << mp << endl; cout << "ABORTED1" << endl; CALCULATOR->useIntervalArithmetic(b_iv); return;}
+	eo.approximation = APPROXIMATION_EXACT;
+	m2 = mp;
+	CALCULATOR->calculate(&m2, 5000, eo);
+	if(m2.isAborted()) {cout << str << " => " << mp << endl; cout << "ABORTED2" << endl; CALCULATOR->useIntervalArithmetic(b_iv); return;}
+	eo.approximation = APPROXIMATION_APPROXIMATE;
+	CALCULATOR->calculate(&m2, 5000, eo);
+	if(m2.isAborted()) {cout << str << " => " << mp << endl; cout << "ABORTED3" << endl; CALCULATOR->useIntervalArithmetic(b_iv); return;}
+	if(m1.isNumber() && m2.isNumber()) {
+		rt1++;
+		if(m1 != m2 && m1.print(po) != m2.print(po)) {
+			rt2++;
+			cout << str << " => " << mp << endl;
+			cout << "UNEQUAL1: " << m1.print(po) << ":" << m2.print(po) << endl;
+		}
+	}
+	cerr << "A" << endl;
+	if(b_iv != CALCULATOR->usesIntervalArithmetic()) {
+		cout << "INTERVAL ARITHMETIC CHANGED1: " << str << " => " << mp << endl;
+		CALCULATOR->useIntervalArithmetic(b_iv);
+	}
+	if(mp.contains(CALCULATOR->v_x)) {
+		rt3++;
+		Number nr(rnd_number(false));
+		if(nr.hasImaginaryPart() && rand() % 2 == 0) nr += Number(rnd_number(false));
+		m1 = mp;
+		m1.replace(CALCULATOR->v_x, nr);
+		CALCULATOR->calculate(&m1, 5000, eo);
+		if(m1.isAborted()) {cout << str << " => " << mp << endl; cout << "ABORTED4: " << nr << endl; CALCULATOR->useIntervalArithmetic(b_iv); return;}
+		m2 = mp;
+		CALCULATOR->v_x->setAssumptions(nr);
+		eo.approximation = APPROXIMATION_EXACT;
+		CALCULATOR->calculate(&m2, 5000, eo);
+		CALCULATOR->v_x->setAssumptions(NULL);
+		if(m2.isAborted()) {cout << str << " => " << mp << endl; cout << "ABORTED5: " << nr << endl; CALCULATOR->useIntervalArithmetic(b_iv); return;}
+		m2.replace(CALCULATOR->v_x, nr);
+		eo.approximation = APPROXIMATION_APPROXIMATE;
+		CALCULATOR->calculate(&m2, 5000, eo);
+		if(m2.isAborted()) {cout << str << " => " << mp << endl; cout << "ABORTED6: " << nr << endl; CALCULATOR->useIntervalArithmetic(b_iv); return;}
+		if(m1.isNumber() && m2.isNumber()) {
+			rt4++;
+			if(m1 != m2 && m1.print(po) != m2.print(po)) {
+				rt5++;
+				cout << str << " => " << mp << ":" << nr << endl;
+				cout << "UNEQUAL2: " << m1.print(po) << ":" << m2.print(po) << endl;
+			}
+		}
+		if(b_iv != CALCULATOR->usesIntervalArithmetic()) {
+			cout << "INTERVAL ARITHMETIC CHANGED3: " << str << " => " << mp << endl;
+			CALCULATOR->useIntervalArithmetic(b_iv);
+		}
+		if(test_interval) {
+			m1 = mp;
+			if(nr.hasImaginaryPart()) nr = rand() % 100 - 50;
+			nr.setPrecision(5);
+			Number nr_iv = nr;
+			nr_iv.precisionToInterval();
+			cerr << "I1: " << nr_iv << endl;
+			v->set(nr_iv);
+			m1.replace(CALCULATOR->v_x, v);
+			m2 = m1;
+			CALCULATOR->useIntervalArithmetic(true);
+			CALCULATOR->calculate(&m1, 5000, eo);
+			if(m1.isAborted()) {cout << str << " => " << mp << endl; cout << "ABORTED9"; CALCULATOR->useIntervalArithmetic(b_iv); return;}
+			CALCULATOR->useIntervalArithmetic(false);
+			v->set(nr);
+			cerr << "I2" << endl;
+			CALCULATOR->calculate(&m2, 5000, eo);
+			CALCULATOR->useIntervalArithmetic(b_iv);
+			if(m2.isAborted()) {cout << str << " => " << mp << endl; cout << "ABORTED10"; CALCULATOR->useIntervalArithmetic(b_iv); return;}
+			if(m1.isNumber() && m2.isNumber() && m1.number().precision(true) > 2) {
+				rt8++;
+				PrintOptions po;
+				po.min_exp = 1;
+				po.max_decimals = 2;
+				
+				po.interval_display = INTERVAL_DISPLAY_SIGNIFICANT_DIGITS;
+				if(m1 != m2) {
+					string si1 = m1.number().imaginaryPart().print(po);
+					string si2 = m2.number().imaginaryPart().print(po);
+					string sr1 = m1.number().realPart().print(po);
+					string sr2 = m2.number().realPart().print(po);
+					if(si1 != si2 && si1 == "0") {
+						nr = m2.number().imaginaryPart();
+						nr.abs();
+						nr.log(10);
+						if(nr < -20) {
+							si2 = "0";
+						}
+					}
+					if(sr1 != sr2 && sr1 == "0") {
+						nr = m2.number().realPart();
+						nr.abs();
+						nr.log(10);
+						if(nr < -20) {
+							sr2 = "0";
+						}
+					}
+					if(si1 != si2 && si2 == "0") {
+						nr = m1.number().imaginaryPart();
+						nr.abs();
+						nr.log(10);
+						if(nr < -20) {
+							si1 = "0";
+						}
+					}
+					if(sr1 != sr2 && sr2 == "0") {
+						nr = m1.number().realPart();
+						nr.abs();
+						nr.log(10);
+						if(nr < -20) {
+							sr1 = "0";
+						}
+					}
+					if((sr1 != sr2 && sr1.length() < 10) || (si1 != si2 && si1.length() < 10)) {
+						po.max_decimals = 1;
+						if(sr1 != sr2) sr2 = m2.number().realPart().print(po);
+						if(si1 != si2) si2 = m2.number().imaginaryPart().print(po);
+						if((sr1 != sr2 && sr1.length() < 10) || (si1 != si2 && si1.length() < 10)) {
+							rt9++;
+							cout << str << " => " << mp << ":" << nr_iv << endl;
+							cout << "UNEQUAL4: " << m1.print(po) << ":" << m2.print(po) << endl;
+						}
+					}
+				}
+			}
+		}
+		
+		if(test_equation) {
+			m1 = mp;
+			cerr << "B" << endl;
+			m1.transform(COMPARISON_EQUALS, m_zero);
+			eo.approximation = APPROXIMATION_EXACT;
+			CALCULATOR->calculate(&m1, 5000, eo);
+			if(m1.isAborted()) {cout << str << " => " << mp << endl; cout << "ABORTED7"; CALCULATOR->useIntervalArithmetic(b_iv); return;}
+			m1.replace(CALCULATOR->v_n, 3);
+			if(m1.isComparison() && m1.comparisonType() == COMPARISON_EQUALS && m1[0] == CALCULATOR->v_x) {
+				m2 = mp;
+				m2.replace(CALCULATOR->v_x, m1[1]);
+				eo.approximation = APPROXIMATION_APPROXIMATE;
+				CALCULATOR->calculate(&m2, 5000, eo);
+				if(m2.isAborted()) {cout << str << " => " << mp << endl; cout << "ABORTED8: " << m1[1] << endl; CALCULATOR->useIntervalArithmetic(b_iv); return;}
+				if(m2.isNumber()) {
+					rt6++;
+					if(m2.number().isNonZero()) {
+						nr = m2.number();
+						nr.abs();
+						nr.log(10);
+						if(nr > -10) {
+							rt7++;
+							cout << str << " => " << mp << endl;
+							cout << "UNEQUAL3: " << m1.print(po) << ":" << m2.print(po) << endl;
+						}
+					}
+				}
+			} else if(!m1.isComparison()) {
+				bool  b = true;
+				if(m1.type() == STRUCT_LOGICAL_AND) {
+					for(size_t i = 0; i < m1.size(); i++) {
+						if(m1[i].isComparison() && m1[i].comparisonType() != COMPARISON_EQUALS) {
+							eo.approximation = APPROXIMATION_APPROXIMATE;
+							CALCULATOR->calculate(&m1[i], 5000, eo);
+							if(m1[i].isAborted()) {cout << str << " => " << mp << endl; cout << "ABORTED9: " << m1[i] << endl; CALCULATOR->useIntervalArithmetic(b_iv); return;}
+							if(m1[i].isZero()) {
+								b = false;
+								break;
+							}
+						}
+					}
+				}
+				if(b) {
+					for(size_t i = 0; i < m1.size(); i++) {
+						if(m1[i].isComparison() && m1[i].comparisonType() == COMPARISON_EQUALS && m1[i][0] == CALCULATOR->v_x) {
+							m2 = mp;
+							m2.replace(CALCULATOR->v_x, m1[i][1]);
+							eo.approximation = APPROXIMATION_APPROXIMATE;
+							CALCULATOR->calculate(&m2, 5000, eo);
+							if(m2.isAborted()) {cout << str << " => " << mp << endl; cout << "ABORTED8: " << m1[i][1] << endl; CALCULATOR->useIntervalArithmetic(b_iv); return;}
+							if(m2.isNumber()) {
+								rt6++;
+								if(m2.number().isNonZero()) {
+									nr = m2.number();
+									nr.abs();
+									nr.log(10);
+									if(nr > -10) {
+										rt7++;
+										cout << str << " => " << mp << endl;
+										cout << "UNEQUAL3: " << m1[i].print(po) << ":" << m2.print(po) << endl;
+									}
+								}
+							}
+						} else if(!m1[i].isComparison()) {
+							b = true;
+							if(m1[i].type() == STRUCT_LOGICAL_AND) {
+								for(size_t i2 = 0; i2 < m1[i].size(); i2++) {
+									if(m1[i][i2].isComparison() && m1[i][i2].comparisonType() != COMPARISON_EQUALS) {
+										eo.approximation = APPROXIMATION_APPROXIMATE;
+										CALCULATOR->calculate(&m1[i][i2], 5000, eo);
+										if(m1[i][i2].isAborted()) {cout << str << " => " << mp << endl; cout << "ABORTED9: " << m1[i][i2] << endl; CALCULATOR->useIntervalArithmetic(b_iv); return;}
+										if(m1[i][i2].isZero()) {
+											b = false;
+											break;
+										}
+									}
+								}
+							}
+							if(b) {
+								for(size_t i2 = 0; i2 < m1[i].size(); i2++) {
+									if(m1[i][i2].isComparison() && m1[i][i2].comparisonType() == COMPARISON_EQUALS && m1[i][i2][0] == CALCULATOR->v_x) {
+										m2 = mp;
+										m2.replace(CALCULATOR->v_x, m1[i][i2][1]);
+										eo.approximation = APPROXIMATION_APPROXIMATE;
+										CALCULATOR->calculate(&m2, 5000, eo);
+										if(m2.isAborted()) {cout << str << " => " << mp << endl; cout << "ABORTED8: " << m1[i][i2][1] << endl; CALCULATOR->useIntervalArithmetic(b_iv); return;}
+										if(m2.isNumber()) {
+											rt6++;
+											if(m2.number().isNonZero()) {
+												nr = m2.number();
+												nr.abs();
+												nr.log(10);
+												if(nr > -10) {
+													rt7++;
+													cout << str << " => " << mp << endl;
+													cout << "UNEQUAL3: " << m1[i][i2].print(po) << ":" << m2.print(po) << endl;
+												}
+											}
+										}
+									}
+								}
+							}
+						}
+					}
+				}
+				if(b_iv != CALCULATOR->usesIntervalArithmetic()) {
+					cout << "INTERVAL ARITHMETIC CHANGED5: " << str << " => " << mp << endl;
+					CALCULATOR->useIntervalArithmetic(b_iv);
+				}
+			}
+		}
+	}
+	string str2 = rnd_expression(allow_unknowns, allow_functions, 6, 5, allow_unit, allow_variable);
+	str.insert(0, "(");
+	str += ") / (";
+	str += str2;
+	str += ")";
+	MathStructure mp2;
+	CALCULATOR->parse(&mp2, str2, eo.parse_options);
+	mp /= mp2;
+	eo.approximation = APPROXIMATION_APPROXIMATE;
+	m1 = mp;
+	cerr << mp << endl;
+	CALCULATOR->calculate(&m1, 5000, eo);
+	if(m1.isAborted()) {cout << str << " => " << mp << endl; cout << "ABORTED1" << endl; CALCULATOR->useIntervalArithmetic(b_iv); return;}
+	eo.approximation = APPROXIMATION_EXACT;
+	m2 = mp;
+	CALCULATOR->calculate(&m2, 5000, eo);
+	if(m2.isAborted()) {cout << str << " => " << mp << endl; cout << "ABORTED2" << endl; CALCULATOR->useIntervalArithmetic(b_iv); return;}
+	eo.approximation = APPROXIMATION_APPROXIMATE;
+	CALCULATOR->calculate(&m2, 5000, eo);
+	if(m2.isAborted()) {cout << str << " => " << mp << endl; cout << "ABORTED3" << endl; CALCULATOR->useIntervalArithmetic(b_iv); return;}
+	if(m1.isNumber() && m2.isNumber()) {
+		rt1++;
+		if(m1 != m2 && m1.print(po) != m2.print(po)) {
+			rt2++;
+			cout << str << " => " << mp << endl;
+			cout << "UNEQUAL1: " << m1.print(po) << ":" << m2.print(po) << endl;
+		}
+	}
+	if(b_iv != CALCULATOR->usesIntervalArithmetic()) {
+		cout << "INTERVAL ARITHMETIC CHANGED12 " << str << " => " << mp << endl;
+		CALCULATOR->useIntervalArithmetic(b_iv);
+	}
+	cerr << "C" << endl;
+	if(mp.contains(CALCULATOR->v_x)) {
+		rt3++;
+		Number nr(rnd_number(false));
+		if(nr.hasImaginaryPart() && rand() % 2 == 0) nr += Number(rnd_number(false));
+		m1 = mp;
+		m1.replace(CALCULATOR->v_x, nr);
+		CALCULATOR->calculate(&m1, 5000, eo);
+		if(m1.isAborted()) {cout << str << " => " << mp << endl; cout << "ABORTED4: " << nr << endl; CALCULATOR->useIntervalArithmetic(b_iv); return;}
+		m2 = mp;
+		eo.approximation = APPROXIMATION_EXACT;
+		CALCULATOR->v_x->setAssumptions(nr);
+		CALCULATOR->calculate(&m2, 5000, eo);
+		CALCULATOR->v_x->setAssumptions(NULL);
+		if(m2.isAborted()) {cout << str << " => " << mp << endl; cout << "ABORTED5: " << nr << endl; CALCULATOR->useIntervalArithmetic(b_iv); return;}
+		m2.replace(CALCULATOR->v_x, nr);
+		eo.approximation = APPROXIMATION_APPROXIMATE;
+		CALCULATOR->calculate(&m2, 5000, eo);
+		if(m2.isAborted()) {cout << str << " => " << mp << endl; cout << "ABORTED6: " << nr << endl; CALCULATOR->useIntervalArithmetic(b_iv); return;}
+		if(m1.isNumber() && m2.isNumber()) {
+			rt4++;
+			if(m1 != m2 && m1.print(po) != m2.print(po)) {
+				rt5++;
+				cout << str << " => " << mp << ":" << nr << endl;
+				cout << "UNEQUAL2: " << m1.print(po) << ":" << m2.print(po) << endl;
+			}
+		}
+		if(b_iv != CALCULATOR->usesIntervalArithmetic()) {
+			cout << "INTERVAL ARITHMETIC CHANGED4: " << str << " => " << mp << endl;
+			CALCULATOR->useIntervalArithmetic(b_iv);
+		}
+		if(test_interval) {
+			m1 = mp;
+			if(nr.hasImaginaryPart()) nr = rand() % 100 - 50;
+			nr.setPrecision(5);
+			Number nr_iv = nr;
+			nr_iv.precisionToInterval();
+			cerr << "I1: " << nr_iv << endl;
+			v->set(nr_iv);
+			m1.replace(CALCULATOR->v_x, v);
+			m2 = m1;
+			CALCULATOR->useIntervalArithmetic(true);
+			CALCULATOR->calculate(&m1, 5000, eo);
+			if(m1.isAborted()) {cout << str << " => " << mp << endl; cout << "ABORTED9"; CALCULATOR->useIntervalArithmetic(b_iv); return;}
+			CALCULATOR->useIntervalArithmetic(false);
+			v->set(nr);
+			cerr << "I2" << endl;
+			CALCULATOR->calculate(&m2, 5000, eo);
+			CALCULATOR->useIntervalArithmetic(b_iv);
+			if(m2.isAborted()) {cout << str << " => " << mp << endl; cout << "ABORTED10"; return;}
+			if(m1.isNumber() && m2.isNumber() && m1.number().precision(true) > 2) {
+				rt8++;
+				PrintOptions po;
+				po.min_exp = 1;
+				po.max_decimals = 2;
+				
+				po.interval_display = INTERVAL_DISPLAY_SIGNIFICANT_DIGITS;
+				if(m1 != m2) {
+					string si1 = m1.number().imaginaryPart().print(po);
+					string si2 = m2.number().imaginaryPart().print(po);
+					string sr1 = m1.number().realPart().print(po);
+					string sr2 = m2.number().realPart().print(po);
+					if(si1 != si2 && si1 == "0") {
+						nr = m2.number().imaginaryPart();
+						nr.abs();
+						nr.log(10);
+						if(nr < -20) {
+							si2 = "0";
+						}
+					}
+					if(sr1 != sr2 && sr1 == "0") {
+						nr = m2.number().realPart();
+						nr.abs();
+						nr.log(10);
+						if(nr < -20) {
+							sr2 = "0";
+						}
+					}
+					if(si1 != si2 && si2 == "0") {
+						nr = m1.number().imaginaryPart();
+						nr.abs();
+						nr.log(10);
+						if(nr < -20) {
+							si1 = "0";
+						}
+					}
+					if(sr1 != sr2 && sr2 == "0") {
+						nr = m1.number().realPart();
+						nr.abs();
+						nr.log(10);
+						if(nr < -20) {
+							sr1 = "0";
+						}
+					}
+					if((sr1 != sr2 && sr1.length() < 10) || (si1 != si2 && si1.length() < 10)) {
+						po.max_decimals = 1;
+						if(sr1 != sr2) sr2 = m2.number().realPart().print(po);
+						if(si1 != si2) si2 = m2.number().imaginaryPart().print(po);
+						if((sr1 != sr2 && sr1.length() < 10) || (si1 != si2 && si1.length() < 10)) {
+							rt9++;
+							cout << str << " => " << mp << ":" << nr_iv << endl;
+							cout << "UNEQUAL4: " << m1.print(po) << ":" << m2.print(po) << endl;
+						}
+					}
+				}
+			}
+		}
+	}
+	cerr << "D" << endl;
+	//if(display_errors(true)) cout << str << ":" << str2 << endl;
+}
+
 void speed_test() {
 		
 	/*UserFunction f1("", "", "x^2/3");
@@ -997,6 +1577,7 @@ int main(int argc, char *argv[]) {
 	new Calculator();
 	CALCULATOR->loadGlobalDefinitions();
 	CALCULATOR->loadLocalDefinitions();
+	CALCULATOR->setPrecision(40);
 	
 	EvaluationOptions evalops;
 	/*evalops.approximation = APPROXIMATION_TRY_EXACT;
@@ -1014,8 +1595,9 @@ int main(int argc, char *argv[]) {
 	//evalops.parse_options.parsing_mode = PARSING_MODE_ADAPTIVE;
 	evalops.parse_options.angle_unit = ANGLE_UNIT_RADIANS;
 	/*evalops.parse_options.dot_as_separator = CALCULATOR->default_dot_as_separator;
-	evalops.parse_options.comma_as_separator = false;
-	evalops.mixed_units_conversion = MIXED_UNITS_CONVERSION_DEFAULT;*/
+	evalops.parse_options.comma_as_separator = false;*/
+	evalops.mixed_units_conversion = MIXED_UNITS_CONVERSION_DEFAULT;
+	evalops.auto_post_conversion = POST_CONVERSION_OPTIMAL;
 	
 	/*MathStructure mstruct = CALCULATOR->calculate("atanh(2x^2+5)*x^2", evalops);
 	cout << mstruct.integrate(CALCULATOR->v_x, evalops) << endl;
@@ -1026,8 +1608,25 @@ int main(int argc, char *argv[]) {
 	mstruct.eval(evalops);
 	cout << mstruct << endl;*/
 	//speed_test();
-	test_integration();
+	//test_integration();
 	//test_intervals(true);
+	
+	CALCULATOR->setVariableUnitsEnabled(false);
+	
+	PrintOptions po = CALCULATOR->messagePrintOptions();
+	po.interval_display = INTERVAL_DISPLAY_SIGNIFICANT_DIGITS;
+	CALCULATOR->setMessagePrintOptions(po);
+	
+	v = new KnownVariable("", "v", m_zero);
+
+	//CALCULATOR->defaultAssumptions()->setType(ASSUMPTION_TYPE_NUMBER);
+	CALCULATOR->useIntervalArithmetic();
+	
+	for(size_t i = 0; i <= 10000; i++) {
+		rnd_test(evalops, 4, true, true, true, false, false);
+		if(i % 1000 == 0) cout << endl << rt1 << ":" << rt2 << ":" << rt3 << ":" << rt4 << ":" << rt5 << ":" << rt6 << ":" << rt7 << ":" << rt8 << ":" << rt9 << endl << endl;
+	}
+	cout << endl << endl << "-----------------------------------------" << endl << endl << endl;
 
 	return 0;
 
